'''
Copyright (C) 2021  Alexander "PapaTutuWawa"

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''

class BaseModule:
    def __init__(self, base, config={}, subcommand_table={}):
        self._name = ''
        self._base = base
        self._config = config
        self._subcommand_table = subcommand_table
        
    def set_name(self, name):
        if self._name:
            raise Exception('Name change of module attempted!')

        self._name = name
        
    def get_option(self, key, default=None):
        return self._config.get(key, default)

    # Used for access control
    # Returns True if @jid is allowed to access the command. False otherwise.
    # This is configured by either restrict_local or allowed_domains. restrict_local
    # takes precedence over allowed_domains
    def is_jid_allowed(self, jid):
        only_local = self.get_option('restrict_local')
        if only_local:
            return only_local

        domains = self.get_option('allowed_domains')
        if not domains:
            return True

        return jid.domain in domains
    
    def get_subscriptions_for(self, jid):
        return self._base.get_subscription_manager.get_subscriptions_for(self._name, jid)

    def add_subscription_for(self, jid, keyword):
        return self._base.get_subscription_manager.add_subscription_for(self._name, jid, keyword)

    def remove_subscription_for(self, jid, keyword):
        return self._base.get_subscription_manager.remove_subscription_for(self._name, jid, keyword)

    def is_subscribed_to(self, jid, keyword):
        return self._base.get_subscription_manager.is_subscribed_to(self._name, jid, keyword)

    def _on_command(self, cmd, msg):
        if not self._subcommand_table:
            self.on_command(cmd, msg)
        elif cmd and cmd[0] in self._subcommand_table:
            self._subcommand_table[cmd[0]](cmd[1:], msg)
        else:
            if '*' in self._subcommand_table:
                self._subcommand_table['*'](cmd, msg)
